using System;
using System.Reflection;
using System.Collections;
using System.ComponentModel;

namespace Infragistics.Win.UltraWinTree.Samples
{
	/// <summary>
	/// Class that provides System.Reflection support
	/// </summary>
	public class ReflectionSupport
	{
		#region Member variables
		private object _selectedObject = null;
		private string[] _propertyNames = null;
		#endregion Member variables

		#region Constructor

		/// <summary>
		/// Creates a new instance of the <see cref="ReflectionSupport"/> object.
		/// </summary>
		/// <param name="selectedObject">The object whose members are to be inspected.</param>
		public ReflectionSupport( object selectedObject )
		{
			this._selectedObject = selectedObject;
		}

		#endregion Constructor

		#region Methods

			#region GetPropertyValue
		/// <summary>
		/// Returns the current value of the specified property of the <see cref="SelectedObject"/>
		/// </summary>
		/// <param name="propertyName">The name of the property whose value is to be returned.</param>
		/// <returns>The specified property's value.</returns>
		public object GetPropertyValue( string propertyName )
		{
			Type type = this._selectedObject.GetType();
			PropertyInfo propInfo = null;
			try
			{
				propInfo = type.GetProperty( propertyName );
			}
			catch
			{
				return null;
			}

			if ( propInfo != null )
			{
				//	Some properties require a parameter, but for the sake
				//	of simplicity, we will not display those properties
				try
				{
					object val = propInfo.GetValue( this._selectedObject, null );
					return val;
				}
				catch
				{
					return null;
				}
			}

			return null;
		}
			#endregion GetPropertyValue

			#region GetPropertyDescription
		/// <summary>
		/// Returns the description for the specified property of the <see cref="SelectedObject"/>
		/// </summary>
		/// <param name="propertyName">The name of the property whose description is to be returned.</param>
		/// <returns>A string containing the value of the property's Description attribute.</returns>
		public string GetPropertyDescription( string propertyName )
		{
			try
			{
				Type type = this._selectedObject.GetType();
				PropertyInfo propInfo = type.GetProperty( propertyName );
				if ( propInfo != null )
				{
					object[] attributes = propInfo.GetCustomAttributes( false );

					if ( attributes != null )
					{
						for ( int i = 0; i < attributes.Length; i ++ )
						{
							if ( attributes[i] is DescriptionAttribute )
							{
								DescriptionAttribute desc = attributes[i] as DescriptionAttribute;
								return desc.Description;
							}
						}
					}
				}
			}
			catch
			{
			}

			return string.Empty;
		}
			#endregion GetPropertyDescription

			#region GetPropertyCategory
		/// <summary>
		/// Returns the category name for the specified property of the <see cref="SelectedObject"/>
		/// </summary>
		/// <param name="propertyName">The name of the property whose category name is to be returned.</param>
		/// <returns>A string containing the value of the property's Category attribute.</returns>
		public string GetPropertyCategory( string propertyName )
		{
			try
			{
				Type type = this._selectedObject.GetType();
				PropertyInfo propInfo = type.GetProperty( propertyName );
				if ( propInfo != null )
				{
					object[] attributes = propInfo.GetCustomAttributes( false );

					if ( attributes != null )
					{
						for ( int i = 0; i < attributes.Length; i ++ )
						{
							if ( attributes[i] is CategoryAttribute )
							{
								CategoryAttribute category = attributes[i] as CategoryAttribute;
								return category.Category;
							}
						}
					}
				}
			}
			catch
			{
			}

			return string.Empty;
		}
			#endregion GetPropertyCategory

		#endregion Methods

		#region Properties

			#region SelectedObject
		/// <summary>
		/// Returns the object this instance was created to inspect.
		/// </summary>
		public object SelectedObject
		{
			get{ return this._selectedObject; }
		}
			#endregion SelectedObject

			#region PropertyNames
		/// <summary>
		/// Returns an array of strings which contain the name of each property
		/// publicly exposed by this instance's <see cref="SelectedObject"/>.
		/// </summary>
		public string[] PropertyNames
		{
			get
			{
				if ( this._propertyNames == null )
				{
					Type type = this._selectedObject.GetType();
					PropertyInfo[] properties = type.GetProperties( BindingFlags.Instance | BindingFlags.Public );

					ArrayList props = new ArrayList( properties.Length );
					for ( int i = 0; i < properties.Length; i ++ )
					{
						bool visible = true;
						PropertyInfo propertyInfo = properties[i];
						object[] attributes = propertyInfo.GetCustomAttributes( false );

						if ( attributes != null )
						{
							for ( int j = 0; j < attributes.Length; j ++ )
							{
								DesignerSerializationVisibilityAttribute attr = attributes[j] as DesignerSerializationVisibilityAttribute;
								if ( attr != null && attr.Visibility == DesignerSerializationVisibility.Hidden )
								{
									visible = false;
									break;
								}
							}
						}

						if ( visible )
							props.Add( properties[i].Name );
					}

					this._propertyNames = props.ToArray( typeof(string) ) as string[];
				}

				return this._propertyNames;
			}
		}
			#endregion PropertyNames

		#endregion Properties
	}
}